@@ -10,8 +10,6 @@ from limits.typing import Dict, List, Optional, Tuple
 
 
 class LockableEntry(asyncio.Lock):
-    __slots__ = ["atime", "expiry"]
-
     def __init__(self, expiry: int) -> None:
         self.atime = time.time()
         self.expiry = self.atime + expiry
@@ -170,7 +168,7 @@ class MemoryStorage(Storage, MovingWindowSupport):
         return True
 
     async def reset(self) -> Optional[int]:
-        num_items = len(self.storage)
+        num_items = max(len(self.storage), len(self.events))
         self.storage.clear()
         self.expirations.clear()
         self.events.clear()
